from openerp.osv import osv
from openerp.osv import fields
from openerp.tools.translate import _ 
from datetime import datetime, timedelta
class Course(osv.Model):
    _name = 'openacademy.course'

    def _get_attendee_count(self, cr, uid, ids, name, args, context=None):
        res = {}
        for course in self.browse(cr, uid, ids, context=context):
            res[course.id] = 0
            for session in course.session_ids:
                res[course.id] += len(session.attendee_ids)
            return res

    # /!\ This method is called from the session object!!
    def _get_courses_from_sessions(self, cr, uid, ids, context=None):
        sessions = self.browse(cr, uid, ids, context=context)
        # Return a list of Course ids with only one occurrence of each
        return list(set(sess.course_id.id for sess in sessions))


    _columns = {
            'name' : fields.char(string='Title', size=256, required=True),
            'description' : fields.text(string='Description'),
            # Relational fields
            'attendee_count': fields.function(_get_attendee_count,
                type='integer', string='Attendee Count',
                store={
                    'openacademy.session' :
                    (_get_courses_from_sessions,['attendee_ids'],0)
                    }),
            'responsible_id' : fields.many2one('res.users',
                # 'set null' will reset responsible_id to
                # undefined if responsible was deleted
                ondelete='set null', string='Responsible', select=True),
            # A one2many is the inverse link of a many2one
            'session_ids' : fields.one2many('openacademy.session', 'course_id',
                string='Session'),
            } 

    _sql_constraints = [
            ('name_description_check',
                'CHECK(name <> description)',
                'The title of the course should be different of the description'),
            ('name_unique',
                'UNIQUE(name)',
                'The title must be unique'),
    ]

 # Allow to make a duplicate Course
    def copy(self, cr, uid, id, default, context=None):
#        import pdb,pprint
 #       pdb.set_trace()
        course = self.browse(cr, uid, id, context=context)
        new_name = "Copy of %s" % course.name
        # =like is the original LIKE operator from SQL
        others_count = self.search(cr, uid, [('name', '=like', new_name+'%')],
                count=True, context=context)
        if others_count > 0:
            new_name = "%s (%s)" % (new_name, others_count+1)
        default['name'] = new_name
        default['session_ids'] = []
        return super(Course, self).copy(cr, uid, id, default, context=context)




class Session(osv.Model):
    _name = 'openacademy.session'

    def _determin_hours_from_duration(self, cr, uid, ids, field, arg,
            context=None):
        result = {}
        sessions = self.browse(cr, uid, ids, context=context)
        for session in sessions:
            result[session.id] = (session.duration * 24 if session.duration
                    else 0)
        return result

    def _set_hours(self, cr, uid, id, field, value, arg, context=None):
        if value:
            self.write(cr, uid, id,
                    {'duration' : (value / 24)},
                    context=context)


    def _get_taken_seats_percent(self, seats, attendee_list):
        # Prevent divison-by-zero error
        try:
            # You need at least one float number to get the real value
            return (100.0 * len(attendee_list)) / seats
        except ZeroDivisionError:
            return 0.0

    # The following definition must appear *after* _get_taken_seats_percent
    # because it's called during the process
    def _taken_seats_percent(self, cr, uid, ids, field, arg, context=None):
        # Compute the percentage of seats that are booked
        context = context or {}
        result = {}.fromkeys(ids,0.0)
        for session in self.browse(cr, uid, ids, context=context):
            result[session.id] = self._get_taken_seats_percent(
                    session.seats, session.attendee_ids)
        print '\nPASO AQUI\n' 
        return result

    def _get_attendee(self, cr, uid, ids, context=None):
        if context is None: context = {}
        res={}
        attendee_obj = self.pool.get('openacademy.attendee')
        for attendee_brw in attendee_obj.browse(cr, uid, ids, context=context):
            res[attendee_brw.session_id.id]=True
        print 'GET ATTENDEE'
        return res.keys()

    def _write_rsp(self, cr, uid, id, name, value, arg, context=None):
        context = context or {}
        assert name == 'taken_seats_percent'
        sql_str = """UPDATE openacademy_session set
                    taken_seats_percent='%s'
                    WHERE
                    id=%d """ % (value, id)
        cr.execute(sql_str)
        return True

    def onchange_taken_seats(self, cr, uid, ids, seats, attendee_ids,context=None):
        context = context or {}
        # Serializes o2m commands into record dictionaries (as if all the o2m
        # records came from the database via a read()), and returns an
        # iterable over these dictionaries.
        attendee_records = self.resolve_2many_commands(
            cr, uid, 'attendee_ids', attendee_ids, ['id'])
        res={}
        if not seats:
            res['value'] = { 'seats':10,
            'taken_seats_percent':self._get_taken_seats_percent(10,attendee_records)
            }
        elif seats < 0:
            res['value'] = { 'seats':10,
            'taken_seats_percent':self._get_taken_seats_percent(10,attendee_records)
            }
            res['warning']={   'title': _('Warning'),
                                'message' : _('You cannot have negative seats')
                                }
        else:
            res['value'] = {
                            'taken_seats_percent':self._get_taken_seats_percent(seats,attendee_records)
                            }
        return res
    

    def _check_instructor_not_in_attendees(self, cr, uid, ids, context=None):
        context = context or {}
        res = [True]
        for session in self.browse(cr, uid, ids, context):
            partners = [att.partner_id for att in session.attendee_ids]
            if session.instructor_id in partners:
                res.append(False)
        return all(res)

    def _determin_end_date(self, cr, uid, ids, field, arg, context=None):
        result = {}
        for session in self.browse(cr, uid, ids, context=context):
            if session.start_date and session.duration:
                # Get a datetime object from session.start_date
                start_date = datetime.strptime(session.start_date, "%Y-%m-%d")
                # Get a timedelta object from session.duration
                duration = timedelta( days=(session.duration - 1) )
                # Calculate the end time
                end_date = start_date + duration
                # Render in format YYYY-MM-DD
                result[session.id] = end_date.strftime("%Y-%m-%d")
            else:
                # Tip: In case session.start_date exists but session.duration
                #   not end_date will be equal to start_date
                result[session.id] = session.start_date
        return result

    # Beware that fnct_inv take a unique id and not a list of id

    def _set_end_date(self, cr, uid, id, field, value, arg, context=None):
        session = self.browse(cr, uid, id, context=context)
        if session.start_date and value:
            start_date = datetime.strptime(session.start_date, "%Y-%m-%d")
            end_date = datetime.strptime(value[:10], "%Y-%m-%d")
            duration = end_date - start_date
            self.write(cr, uid, id, {'duration' : (duration.days + 1)},
                context=context)

  
    def _get_attendee_count(self, cr, uid, ids, name, args, context=None):
        res = {}
        for session in self.browse(cr, uid, ids, context=context):
            res[session.id] = len(session.attendee_ids)
        return res
  
    _columns = {
            'name' : fields.char(string='Name', size=256, required=True),
            'start_date' : fields.date(string='Start date'),
            'duration' : fields.float(string='Duration', digits=(6,2),
                help='Duration in days'),
            'seats':fields.property('openacademy.session',
                                type='integer',
                                string='Seats number',
                                method=True,
                                view_load=True,
                                required=True,),
            'active' : fields.boolean("Active"),
            'color' : fields.integer('Color'),
            'state' : fields.selection([('draft','Draft'),
                ('confirmed','Confirmed'),
                ('done','Done')], string="State"),
            'instructor_id' : fields.many2one('res.partner', string='Instructor',
                domain=['|',('instructor','=',True),
                    ('category_id.name','ilike','Teacher')]),
            'taken_seats_percent' : fields.function(_taken_seats_percent,
                type='float', string='Taken Seats',
                store = {
                'openacademy.attendee':(_get_attendee,[],15),
                'openacademy.session':(lambda self, cr, uid, ids,c={}: ids,['seats', 'attendee_ids'],30),
                },fnct_inv=_write_rsp),
            'attendee_count': fields.function(_get_attendee_count,
                type='integer', string='Attendee Count', store=True),
            'end_date' : fields.function(_determin_end_date,
                fnct_inv=_set_end_date, type='date', string="End Date"),
            'hours' : fields.function(_determin_hours_from_duration,
                fnct_inv=_set_hours, type='float', string="Hours"),
            'course_id' : fields.many2one('openacademy.course',
                # 'cascade' will destroy session if course_id was deleted
                ondelete='cascade', string='Course', required=True),
            'attendee_ids' : fields.one2many('openacademy.attendee', 'session_id',
                string="Attendees"),
            }

    _constraints = [
            (_check_instructor_not_in_attendees,
                "The instructor can not be also an attendee!",
                ['instructor_id','attendee_ids']),
    ] 

    _defaults = {
        # You can give a function or a lambda for default value. OpenERP will
        # automatically call it and get its return value at each record
        # creation. 'today' is a method of the object 'date' of module 'fields'
        # that returns -in the right timezone- the current date in the OpenERP
        # format
        'start_date' : fields.date.today,
      #  'start_date' : time.srtftime('%Y-%m-%d'),
      #  'start_date' : lambda *a: time.srtftime('%Y-%m-%d'),
        # Beware that is not the same as:
        # 'start_date' : fields.date.today(),
        # which actually call the method at OpenERP startup!
        'active' : True,
        'state': 'draft',
    }





class Attendee(osv.Model):
    _name = 'openacademy.attendee'
    # _rec_name redefine the field to get when seeing the record in
    # another object. In this case, the partner's name will be printed
    _rec_name = 'partner_id'
    # (useful when no field 'name' has been defined)
    _columns = {
            # There is only relational fields
            'partner_id': fields.many2one('res.partner', string='Partner',
                required=True, ondelete='cascade'),
            'session_id': fields.many2one('openacademy.session', string='Session',
            required=True, ondelete='cascade'),
            'course_id': fields.related('session_id', 
                                    'course_id', 
                                    type='many2one', 
                                    relation='openacademy.course', 
                                    string='Course', 
                                    readonly=True,
                                    store=False),
            'tutor_id': fields.reference('Tutor', 
                                        [('res.partner','Partner'),
                                         ('res.users','Users'),
                                         ('res.company', 'Companies')],
                                        size=128,
                                        required=False),


            }

    _sql_constraints = [
            ('partner_session_unique',
                'UNIQUE(partner_id, session_id)',
                'You can not insert the same attendee multiple times!'),
    ]

